      SUBROUTINE LMDIF(FCN,M,N,X,FVEC,FTOL,XTOL,GTOL,MAXFEV,EPSFCN,DIAG,
     &                 MODE,FACTOR,NPRINT,INFO,NFEV,FJAC,LDFJAC,IPVT,
     &                 QTF,WA1,WA2,WA3,WA4)
      INTEGER M,N,MAXFEV,MODE,NPRINT,INFO,NFEV,LDFJAC
      INTEGER IPVT(N)
      REAL FTOL,XTOL,GTOL,EPSFCN,FACTOR
      REAL X(N),FVEC(M),DIAG(N),FJAC(LDFJAC,N),QTF(N),WA1(N),WA2(N),
     &     WA3(N),WA4(M)
      EXTERNAL FCN
c     **********
c
c     subroutine lmdif
c
c     the purpose of lmdif is to minimize the sum of the squares of
c     m nonlinear functions in n variables by a modification of
c     the levenberg-marquardt algorithm. the user must provide a
c     subroutine which calculates the functions. the jacobian is
c     then calculated by a forward-difference approximation.
c
c     the subroutine statement is
c
c       subroutine lmdif(fcn,m,n,x,fvec,ftol,xtol,gtol,maxfev,epsfcn,
c                        diag,mode,factor,nprint,info,nfev,fjac,
c                        ldfjac,ipvt,qtf,wa1,wa2,wa3,wa4)
c
c     where
c
c       fcn is the name of the user-supplied subroutine which
c         calculates the functions. fcn must be declared
c         in an external statement in the user calling
c         program, and should be written as follows.
c
c         subroutine fcn(m,n,x,fvec,iflag)
c         integer m,n,iflag
c         real x(n),fvec(m)
c         ----------
c         calculate the functions at x and
c         return this vector in fvec.
c         ----------
c         return
c         end
c
c         the value of iflag should not be changed by fcn unless
c         the user wants to terminate execution of lmdif.
c         in this case set iflag to a negative integer.
c
c       m is a positive integer input variable set to the number
c         of functions.
c
c       n is a positive integer input variable set to the number
c         of variables. n must not exceed m.
c
c       x is an array of length n. on input x must contain
c         an initial estimate of the solution vector. on output x
c         contains the final estimate of the solution vector.
c
c       fvec is an output array of length m which contains
c         the functions evaluated at the output x.
c
c       ftol is a nonnegative input variable. termination
c         occurs when both the actual and predicted relative
c         reductions in the sum of squares are at most ftol.
c         therefore, ftol measures the relative error desired
c         in the sum of squares.
c
c       xtol is a nonnegative input variable. termination
c         occurs when the relative error between two consecutive
c         iterates is at most xtol. therefore, xtol measures the
c         relative error desired in the approximate solution.
c
c       gtol is a nonnegative input variable. termination
c         occurs when the cosine of the angle between fvec and
c         any column of the jacobian is at most gtol in absolute
c         value. therefore, gtol measures the orthogonality
c         desired between the function vector and the columns
c         of the jacobian.
c
c       maxfev is a positive integer input variable. termination
c         occurs when the number of calls to fcn is at least
c         maxfev by the end of an iteration.
c
c       epsfcn is an input variable used in determining a suitable
c         step length for the forward-difference approximation. this
c         approximation assumes that the relative errors in the
c         functions are of the order of epsfcn. if epsfcn is less
c         than the machine precision, it is assumed that the relative
c         errors in the functions are of the order of the machine
c         precision.
c
c       diag is an array of length n. if mode = 1 (see
c         below), diag is internally set. if mode = 2, diag
c         must contain positive entries that serve as
c         multiplicative scale factors for the variables.
c
c       mode is an integer input variable. if mode = 1, the
c         variables will be scaled internally. if mode = 2,
c         the scaling is specified by the input diag. other
c         values of mode are equivalent to mode = 1.
c
c       factor is a positive input variable used in determining the
c         initial step bound. this bound is set to the product of
c         factor and the euclidean norm of diag*x if nonzero, or else
c         to factor itself. in most cases factor should lie in the
c         interval (.1,100.). 100. is a generally recommended value.
c
c       nprint is an integer input variable that enables controlled
c         printing of iterates if it is positive. in this case,
c         fcn is called with iflag = 0 at the beginning of the first
c         iteration and every nprint iterations thereafter and
c         immediately prior to return, with x and fvec available
c         for printing. if nprint is not positive, no special calls
c         of fcn with iflag = 0 are made.
c
c       info is an integer output variable. if the user has
c         terminated execution, info is set to the (negative)
c         value of iflag. see description of fcn. otherwise,
c         info is set as follows.
c
c         info = 0  improper input parameters.
c
c         info = 1  both actual and predicted relative reductions
c                   in the sum of squares are at most ftol.
c
c         info = 2  relative error between two consecutive iterates
c                   is at most xtol.
c
c         info = 3  conditions for info = 1 and info = 2 both hold.
c
c         info = 4  the cosine of the angle between fvec and any
c                   column of the jacobian is at most gtol in
c                   absolute value.
c
c         info = 5  number of calls to fcn has reached or
c                   exceeded maxfev.
c
c         info = 6  ftol is too small. no further reduction in
c                   the sum of squares is possible.
c
c         info = 7  xtol is too small. no further improvement in
c                   the approximate solution x is possible.
c
c         info = 8  gtol is too small. fvec is orthogonal to the
c                   columns of the jacobian to machine precision.
c
c       nfev is an integer output variable set to the number of
c         calls to fcn.
c
c       fjac is an output m by n array. the upper n by n submatrix
c         of fjac contains an upper triangular matrix r with
c         diagonal elements of nonincreasing magnitude such that
c
c                t     t           t
c               p *(jac *jac)*p = r *r,
c
c         where p is a permutation matrix and jac is the final
c         calculated jacobian. column j of p is column ipvt(j)
c         (see below) of the identity matrix. the lower trapezoidal
c         part of fjac contains information generated during
c         the computation of r.
c
c       ldfjac is a positive integer input variable not less than m
c         which specifies the leading dimension of the array fjac.
c
c       ipvt is an integer output array of length n. ipvt
c         defines a permutation matrix p such that jac*p = q*r,
c         where jac is the final calculated jacobian, q is
c         orthogonal (not stored), and r is upper triangular
c         with diagonal elements of nonincreasing magnitude.
c         column j of p is column ipvt(j) of the identity matrix.
c
c       qtf is an output array of length n which contains
c         the first n elements of the vector (q transpose)*fvec.
c
c       wa1, wa2, and wa3 are work arrays of length n.
c
c       wa4 is a work array of length m.
c
c     subprograms called
c
c       user-supplied ...... fcn
c
c       minpack-supplied ... spmpar,enorm,fdjac2,lmpar,qrfac
c
c       fortran-supplied ... abs,amax1,amin1,sqrt,mod
c
c     argonne national laboratory. minpack project. march 1980.
c     burton s. garbow, kenneth e. hillstrom, jorge j. more
c
c     **********
      INTEGER I,IFLAG,ITER,J,L
      REAL ACTRED,DELTA,DIRDER,EPSMCH,FNORM,FNORM1,GNORM,ONE,PAR,PNORM,
     &     PRERED,P1,P5,P25,P75,P0001,RATIO,SUM,TEMP,TEMP1,TEMP2,XNORM,
     &     ZERO
      REAL SPMPAR,ENORM
      SAVE ONE,P1,P5,P25,P75,P0001,ZERO
      DATA ONE,P1,P5,P25,P75,P0001,ZERO/1.0E0,1.0E-1,5.0E-1,2.5E-1,
     &     7.5E-1,1.0E-4,0.0E0/
c
c     epsmch is the machine precision.
c
      EPSMCH = SPMPAR(1)
c
      INFO = 0
      IFLAG = 0
      NFEV = 0
c
c     check the input parameters for errors.
c
      IF ( N.GT.0 .AND. M.GE.N .AND. LDFJAC.GE.M .AND. 
     &     FTOL.GE.ZERO .AND. XTOL.GE.ZERO .AND. GTOL.GE.ZERO .AND. 
     &     MAXFEV.GT.0 .AND. FACTOR.GT.ZERO ) THEN
        IF ( MODE.EQ.2 ) THEN
          DO 20 J = 1,N
            IF ( DIAG(J).LE.ZERO ) GOTO 100
 20       CONTINUE
        ENDIF
c
c     evaluate the function at the starting point
c     and calculate its norm.
c
        IFLAG = 1
        CALL FCN(M,N,X,FVEC,IFLAG)
        NFEV = 1
        IF ( IFLAG.GE.0 ) THEN
          FNORM = ENORM(M,FVEC)
c
c     initialize levenberg-marquardt parameter and iteration counter.
c
          PAR = ZERO
          ITER = 1
c
c     beginning of the outer loop.
c
c
c        calculate the jacobian matrix.
c
 40       IFLAG = 2
          CALL FDJAC2(FCN,M,N,X,FVEC,FJAC,LDFJAC,IFLAG,EPSFCN,WA4)
          NFEV = NFEV + N
          IF ( IFLAG.GE.0 ) THEN
c
c        if requested, call fcn to enable printing of iterates.
c
            IF ( NPRINT.GT.0 ) THEN
              IFLAG = 0
              IF ( MOD(ITER-1,NPRINT).EQ.0 ) CALL FCN(M,N,X,FVEC,IFLAG)
              IF ( IFLAG.LT.0 ) GOTO 100
            ENDIF
c
c        compute the qr factorization of the jacobian.
c
            CALL QRFAC(M,N,FJAC,LDFJAC,.TRUE.,IPVT,N,WA1,WA2,WA3)
c
c        on the first iteration and if mode is 1, scale according
c        to the norms of the columns of the initial jacobian.
c
            IF ( ITER.EQ.1 ) THEN
              IF ( MODE.NE.2 ) THEN
                DO 42 J = 1,N
                  DIAG(J) = WA2(J)
                  IF ( WA2(J).EQ.ZERO ) DIAG(J) = ONE
 42             CONTINUE
              ENDIF
c
c        on the first iteration, calculate the norm of the scaled x
c        and initialize the step bound delta.
c
              DO 45 J = 1,N
                WA3(J) = DIAG(J)*X(J)
 45           CONTINUE
              XNORM = ENORM(N,WA3)
              DELTA = FACTOR*XNORM
              IF ( DELTA.EQ.ZERO ) DELTA = FACTOR
            ENDIF
c
c        form (q transpose)*fvec and store the first n components in
c        qtf.
c
            DO 50 I = 1,M
              WA4(I) = FVEC(I)
 50         CONTINUE
            DO 60 J = 1,N
              IF ( FJAC(J,J).NE.ZERO ) THEN
                SUM = ZERO
                DO 52 I = J,M
                  SUM = SUM + FJAC(I,J)*WA4(I)
 52             CONTINUE
                TEMP = -SUM/FJAC(J,J)
                DO 54 I = J,M
                  WA4(I) = WA4(I) + FJAC(I,J)*TEMP
 54             CONTINUE
              ENDIF
              FJAC(J,J) = WA1(J)
              QTF(J) = WA4(J)
 60         CONTINUE
c
c        compute the norm of the scaled gradient.
c
            GNORM = ZERO
            IF ( FNORM.NE.ZERO ) THEN
              DO 65 J = 1,N
                L = IPVT(J)
                IF ( WA2(L).NE.ZERO ) THEN
                  SUM = ZERO
                  DO 62 I = 1,J
                    SUM = SUM + FJAC(I,J)*(QTF(I)/FNORM)
 62               CONTINUE
                  GNORM = AMAX1(GNORM,ABS(SUM/WA2(L)))
                ENDIF
 65           CONTINUE
            ENDIF
c
c        test for convergence of the gradient norm.
c
            IF ( GNORM.LE.GTOL ) INFO = 4
            IF ( INFO.EQ.0 ) THEN
c
c        rescale if necessary.
c
              IF ( MODE.NE.2 ) THEN
                DO 66 J = 1,N
                  DIAG(J) = AMAX1(DIAG(J),WA2(J))
 66             CONTINUE
c
c        beginning of the inner loop.
c
              ENDIF
c
c           determine the levenberg-marquardt parameter.
c
 70           CALL LMPAR(N,FJAC,LDFJAC,IPVT,DIAG,QTF,DELTA,PAR,WA1,WA2,
     &                   WA3,WA4)
c
c           store the direction p and x + p. calculate the norm of p.
c
              DO 75 J = 1,N
                WA1(J) = -WA1(J)
                WA2(J) = X(J) + WA1(J)
                WA3(J) = DIAG(J)*WA1(J)
 75           CONTINUE
              PNORM = ENORM(N,WA3)
c
c           on the first iteration, adjust the initial step bound.
c
              IF ( ITER.EQ.1 ) DELTA = AMIN1(DELTA,PNORM)
c
c           evaluate the function at x + p and calculate its norm.
c
              IFLAG = 1
              CALL FCN(M,N,WA2,WA4,IFLAG)
              NFEV = NFEV + 1
              IF ( IFLAG.GE.0 ) THEN
                FNORM1 = ENORM(M,WA4)
c
c           compute the scaled actual reduction.
c
                ACTRED = -ONE
                IF ( P1*FNORM1.LT.FNORM ) ACTRED = ONE - (FNORM1/FNORM)
     &               **2
c
c           compute the scaled predicted reduction and
c           the scaled directional derivative.
c
                DO 78 J = 1,N
                  WA3(J) = ZERO
                  L = IPVT(J)
                  TEMP = WA1(L)
                  DO 76 I = 1,J
                    WA3(I) = WA3(I) + FJAC(I,J)*TEMP
 76               CONTINUE
 78             CONTINUE
                TEMP1 = ENORM(N,WA3)/FNORM
                TEMP2 = (SQRT(PAR)*PNORM)/FNORM
                PRERED = TEMP1**2 + TEMP2**2/P5
                DIRDER = -(TEMP1**2+TEMP2**2)
c
c           compute the ratio of the actual to the predicted
c           reduction.
c
                RATIO = ZERO
                IF ( PRERED.NE.ZERO ) RATIO = ACTRED/PRERED
c
c           update the step bound.
c
                IF ( RATIO.LE.P25 ) THEN
                  IF ( ACTRED.GE.ZERO ) TEMP = P5
                  IF ( ACTRED.LT.ZERO )
     &                 TEMP = P5*DIRDER/(DIRDER+P5*ACTRED)
                  IF ( P1*FNORM1.GE.FNORM .OR. TEMP.LT.P1 ) TEMP = P1
                  DELTA = TEMP*AMIN1(DELTA,PNORM/P1)
                  PAR = PAR/TEMP
                ELSEIF ( PAR.EQ.ZERO .OR. RATIO.GE.P75 ) THEN
                  DELTA = PNORM/P5
                  PAR = P5*PAR
                ENDIF
c
c           test for successful iteration.
c
                IF ( RATIO.GE.P0001 ) THEN
c
c           successful iteration. update x, fvec, and their norms.
c
                  DO 80 J = 1,N
                    X(J) = WA2(J)
                    WA2(J) = DIAG(J)*X(J)
 80               CONTINUE
                  DO 82 I = 1,M
                    FVEC(I) = WA4(I)
 82               CONTINUE
                  XNORM = ENORM(N,WA2)
                  FNORM = FNORM1
                  ITER = ITER + 1
                ENDIF
c
c           tests for convergence.
c
                IF ( ABS(ACTRED).LE.FTOL .AND. PRERED.LE.FTOL .AND. 
     &               P5*RATIO.LE.ONE ) INFO = 1
                IF ( DELTA.LE.XTOL*XNORM ) INFO = 2
                IF ( ABS(ACTRED).LE.FTOL .AND. PRERED.LE.FTOL .AND. 
     &               P5*RATIO.LE.ONE .AND. INFO.EQ.2 ) INFO = 3
                IF ( INFO.EQ.0 ) THEN
c
c           tests for termination and stringent tolerances.
c
                  IF ( NFEV.GE.MAXFEV ) INFO = 5
                  IF ( ABS(ACTRED).LE.EPSMCH .AND. 
     &                 PRERED.LE.EPSMCH .AND. P5*RATIO.LE.ONE ) INFO = 6
                  IF ( DELTA.LE.EPSMCH*XNORM ) INFO = 7
                  IF ( GNORM.LE.EPSMCH ) INFO = 8
                  IF ( INFO.EQ.0 ) THEN
c
c           end of the inner loop. repeat if iteration unsuccessful.
c
c
c        end of the outer loop.
c
                    IF ( RATIO.GE.P0001 ) GOTO 40
                    GOTO 70
                  ENDIF
                ENDIF
              ENDIF
            ENDIF
          ENDIF
        ENDIF
      ENDIF
c
c     termination, either normal or user imposed.
c
 100  IF ( IFLAG.LT.0 ) INFO = IFLAG
      IFLAG = 0
      IF ( NPRINT.GT.0 ) CALL FCN(M,N,X,FVEC,IFLAG)
      RETURN
c
c     last card of subroutine lmdif.
c
      END
