 
      SUBROUTINE LMPAR(N,R,LDR,IPVT,DIAG,QTB,DELTA,PAR,X,SDIAG,WA1,WA2)
      INTEGER N,LDR
      INTEGER IPVT(N)
      REAL DELTA,PAR
      REAL R(LDR,N),DIAG(N),QTB(N),X(N),SDIAG(N),WA1(N),WA2(N)
c     **********
c
c     subroutine lmpar
c
c     given an m by n matrix a, an n by n nonsingular diagonal
c     matrix d, an m-vector b, and a positive number delta,
c     the problem is to determine a value for the parameter
c     par such that if x solves the system
c
c           a*x = b ,     sqrt(par)*d*x = 0 ,
c
c     in the least squares sense, and dxnorm is the euclidean
c     norm of d*x, then either par is zero and
c
c           (dxnorm-delta) .le. 0.1*delta ,
c
c     or par is positive and
c
c           abs(dxnorm-delta) .le. 0.1*delta .
c
c     this subroutine completes the solution of the problem
c     if it is provided with the necessary information from the
c     qr factorization, with column pivoting, of a. that is, if
c     a*p = q*r, where p is a permutation matrix, q has orthogonal
c     columns, and r is an upper triangular matrix with diagonal
c     elements of nonincreasing magnitude, then lmpar expects
c     the full upper triangle of r, the permutation matrix p,
c     and the first n components of (q transpose)*b. on output
c     lmpar also provides an upper triangular matrix s such that
c
c            t   t                   t
c           p *(a *a + par*d*d)*p = s *s .
c
c     s is employed within lmpar and may be of separate interest.
c
c     only a few iterations are generally needed for convergence
c     of the algorithm. if, however, the limit of 10 iterations
c     is reached, then the output par will contain the best
c     value obtained so far.
c
c     the subroutine statement is
c
c       subroutine lmpar(n,r,ldr,ipvt,diag,qtb,delta,par,x,sdiag,
c                        wa1,wa2)
c
c     where
c
c       n is a positive integer input variable set to the order of r.
c
c       r is an n by n array. on input the full upper triangle
c         must contain the full upper triangle of the matrix r.
c         on output the full upper triangle is unaltered, and the
c         strict lower triangle contains the strict upper triangle
c         (transposed) of the upper triangular matrix s.
c
c       ldr is a positive integer input variable not less than n
c         which specifies the leading dimension of the array r.
c
c       ipvt is an integer input array of length n which defines the
c         permutation matrix p such that a*p = q*r. column j of p
c         is column ipvt(j) of the identity matrix.
c
c       diag is an input array of length n which must contain the
c         diagonal elements of the matrix d.
c
c       qtb is an input array of length n which must contain the first
c         n elements of the vector (q transpose)*b.
c
c       delta is a positive input variable which specifies an upper
c         bound on the euclidean norm of d*x.
c
c       par is a nonnegative variable. on input par contains an
c         initial estimate of the levenberg-marquardt parameter.
c         on output par contains the final estimate.
c
c       x is an output array of length n which contains the least
c         squares solution of the system a*x = b, sqrt(par)*d*x = 0,
c         for the output par.
c
c       sdiag is an output array of length n which contains the
c         diagonal elements of the upper triangular matrix s.
c
c       wa1 and wa2 are work arrays of length n.
c
c     subprograms called
c
c       minpack-supplied ... spmpar,enorm,qrsolv
c
c       fortran-supplied ... abs,amax1,amin1,sqrt
c
c     argonne national laboratory. minpack project. march 1980.
c     burton s. garbow, kenneth e. hillstrom, jorge j. more
c
c     **********
      INTEGER I,ITER,J,JM1,JP1,K,L,NSING
      REAL DXNORM,DWARF,FP,GNORM,PARC,PARL,PARU,P1,P001,SUM,TEMP,ZERO
      REAL SPMPAR,ENORM
      SAVE P1,P001,ZERO
      DATA P1,P001,ZERO/1.0E-1,1.0E-3,0.0E0/
c
c     dwarf is the smallest positive magnitude.
c
      DWARF = SPMPAR(2)
c
c     compute and store in x the gauss-newton direction. if the
c     jacobian is rank-deficient, obtain a least squares solution.
c
      NSING = N
      DO 100 J = 1,N
        WA1(J) = QTB(J)
        IF ( R(J,J).EQ.ZERO .AND. NSING.EQ.N ) NSING = J - 1
        IF ( NSING.LT.N ) WA1(J) = ZERO
 100  CONTINUE
      IF ( NSING.GE.1 ) THEN
        DO 150 K = 1,NSING
          J = NSING - K + 1
          WA1(J) = WA1(J)/R(J,J)
          TEMP = WA1(J)
          JM1 = J - 1
          IF ( JM1.GE.1 ) THEN
            DO 110 I = 1,JM1
              WA1(I) = WA1(I) - R(I,J)*TEMP
 110        CONTINUE
          ENDIF
 150    CONTINUE
      ENDIF
      DO 200 J = 1,N
        L = IPVT(J)
        X(L) = WA1(J)
 200  CONTINUE
c
c     initialize the iteration counter.
c     evaluate the function at the origin, and test
c     for acceptance of the gauss-newton direction.
c
      ITER = 0
      DO 300 J = 1,N
        WA2(J) = DIAG(J)*X(J)
 300  CONTINUE
      DXNORM = ENORM(N,WA2)
      FP = DXNORM - DELTA
      IF ( FP.GT.P1*DELTA ) THEN
c
c     if the jacobian is not rank deficient, the newton
c     step provides a lower bound, parl, for the zero of
c     the function. otherwise set this bound to zero.
c
        PARL = ZERO
        IF ( NSING.GE.N ) THEN
          DO 320 J = 1,N
            L = IPVT(J)
            WA1(J) = DIAG(L)*(WA2(L)/DXNORM)
 320      CONTINUE
          DO 340 J = 1,N
            SUM = ZERO
            JM1 = J - 1
            IF ( JM1.GE.1 ) THEN
              DO 325 I = 1,JM1
                SUM = SUM + R(I,J)*WA1(I)
 325          CONTINUE
            ENDIF
            WA1(J) = (WA1(J)-SUM)/R(J,J)
 340      CONTINUE
          TEMP = ENORM(N,WA1)
          PARL = ((FP/DELTA)/TEMP)/TEMP
        ENDIF
c
c     calculate an upper bound, paru, for the zero of the function.
c
        DO 400 J = 1,N
          SUM = ZERO
          DO 360 I = 1,J
            SUM = SUM + R(I,J)*QTB(I)
 360      CONTINUE
          L = IPVT(J)
          WA1(J) = SUM/DIAG(L)
 400    CONTINUE
        GNORM = ENORM(N,WA1)
        PARU = GNORM/DELTA
        IF ( PARU.EQ.ZERO ) PARU = DWARF/AMIN1(DELTA,P1)
c
c     if the input par lies outside of the interval (parl,paru),
c     set par to the closer endpoint.
c
        PAR = AMAX1(PAR,PARL)
        PAR = AMIN1(PAR,PARU)
        IF ( PAR.EQ.ZERO ) PAR = GNORM/DXNORM
c
c     beginning of an iteration.
c
 450    ITER = ITER + 1
c
c        evaluate the function at the current value of par.
c
        IF ( PAR.EQ.ZERO ) PAR = AMAX1(DWARF,P001*PARU)
        TEMP = SQRT(PAR)
        DO 500 J = 1,N
          WA1(J) = TEMP*DIAG(J)
 500    CONTINUE
        CALL QRSOLV(N,R,LDR,IPVT,WA1,QTB,X,SDIAG,WA2)
        DO 550 J = 1,N
          WA2(J) = DIAG(J)*X(J)
 550    CONTINUE
        DXNORM = ENORM(N,WA2)
        TEMP = FP
        FP = DXNORM - DELTA
c
c        if the function is small enough, accept the current value
c        of par. also test for the exceptional cases where parl
c        is zero or the number of iterations has reached 10.
c
        IF ( .NOT.(ABS(FP).LE.P1*DELTA .OR. PARL.EQ.ZERO .AND. 
     &       FP.LE.TEMP .AND. TEMP.LT.ZERO .OR. ITER.EQ.10) ) THEN
c
c        compute the newton correction.
c
          DO 560 J = 1,N
            L = IPVT(J)
            WA1(J) = DIAG(L)*(WA2(L)/DXNORM)
 560      CONTINUE
          DO 580 J = 1,N
            WA1(J) = WA1(J)/SDIAG(J)
            TEMP = WA1(J)
            JP1 = J + 1
            IF ( N.GE.JP1 ) THEN
              DO 565 I = JP1,N
                WA1(I) = WA1(I) - R(I,J)*TEMP
 565          CONTINUE
            ENDIF
 580      CONTINUE
          TEMP = ENORM(N,WA1)
          PARC = ((FP/DELTA)/TEMP)/TEMP
c
c        depending on the sign of the function, update parl or paru.
c
          IF ( FP.GT.ZERO ) PARL = AMAX1(PARL,PAR)
          IF ( FP.LT.ZERO ) PARU = AMIN1(PARU,PAR)
c
c        compute an improved estimate for par.
c
          PAR = AMAX1(PARL,PAR+PARC)
c
c        end of an iteration.
c
          GOTO 450
        ENDIF
      ENDIF
c
c     termination.
c
      IF ( ITER.EQ.0 ) PAR = ZERO
      RETURN
c
c     last card of subroutine lmpar.
c
      END
