      SUBROUTINE QRFAC(M,N,A,LDA,PIVOT,IPVT,LIPVT,RDIAG,ACNORM,WA)
      INTEGER M,N,LDA,LIPVT
      INTEGER IPVT(LIPVT)
      LOGICAL PIVOT
      REAL A(LDA,N),RDIAG(N),ACNORM(N),WA(N)
c     **********
c
c     subroutine qrfac
c
c     this subroutine uses householder transformations with column
c     pivoting (optional) to compute a qr factorization of the
c     m by n matrix a. that is, qrfac determines an orthogonal
c     matrix q, a permutation matrix p, and an upper trapezoidal
c     matrix r with diagonal elements of nonincreasing magnitude,
c     such that a*p = q*r. the householder transformation for
c     column k, k = 1,2,...,min(m,n), is of the form
c
c                           t
c           i - (1/u(k))*u*u
c
c     where u has zeros in the first k-1 positions. the form of
c     this transformation and the method of pivoting first
c     appeared in the corresponding linpack subroutine.
c
c     the subroutine statement is
c
c       subroutine qrfac(m,n,a,lda,pivot,ipvt,lipvt,rdiag,acnorm,wa)
c
c     where
c
c       m is a positive integer input variable set to the number
c         of rows of a.
c
c       n is a positive integer input variable set to the number
c         of columns of a.
c
c       a is an m by n array. on input a contains the matrix for
c         which the qr factorization is to be computed. on output
c         the strict upper trapezoidal part of a contains the strict
c         upper trapezoidal part of r, and the lower trapezoidal
c         part of a contains a factored form of q (the non-trivial
c         elements of the u vectors described above).
c
c       lda is a positive integer input variable not less than m
c         which specifies the leading dimension of the array a.
c
c       pivot is a logical input variable. if pivot is set true,
c         then column pivoting is enforced. if pivot is set false,
c         then no column pivoting is done.
c
c       ipvt is an integer output array of length lipvt. ipvt
c         defines the permutation matrix p such that a*p = q*r.
c         column j of p is column ipvt(j) of the identity matrix.
c         if pivot is false, ipvt is not referenced.
c
c       lipvt is a positive integer input variable. if pivot is false,
c         then lipvt may be as small as 1. if pivot is true, then
c         lipvt must be at least n.
c
c       rdiag is an output array of length n which contains the
c         diagonal elements of r.
c
c       acnorm is an output array of length n which contains the
c         norms of the corresponding columns of the input matrix a.
c         if this information is not needed, then acnorm can coincide
c         with rdiag.
c
c       wa is a work array of length n. if pivot is false, then wa
c         can coincide with rdiag.
c
c     subprograms called
c
c       minpack-supplied ... spmpar,enorm
c
c       fortran-supplied ... amax1,sqrt,min0
c
c     argonne national laboratory. minpack project. march 1980.
c     burton s. garbow, kenneth e. hillstrom, jorge j. more
c
c     **********
      INTEGER I,J,JP1,K,KMAX,MINMN
      REAL AJNORM,EPSMCH,ONE,P05,SUM,TEMP,ZERO
      REAL SPMPAR,ENORM
      SAVE ONE,P05,ZERO
      DATA ONE,P05,ZERO/1.0E0,5.0E-2,0.0E0/
c
c     epsmch is the machine precision.
c
      EPSMCH = SPMPAR(1)
c
c     compute the initial column norms and initialize several arrays.
c
      DO 100 J = 1,N
        ACNORM(J) = ENORM(M,A(1,J))
        RDIAG(J) = ACNORM(J)
        WA(J) = RDIAG(J)
        IF ( PIVOT ) IPVT(J) = J
 100  CONTINUE
c
c     reduce a to r with householder transformations.
c
      MINMN = MIN0(M,N)
      DO 200 J = 1,MINMN
        IF ( PIVOT ) THEN
c
c        bring the column of largest norm into the pivot position.
c
          KMAX = J
          DO 120 K = J,N
            IF ( RDIAG(K).GT.RDIAG(KMAX) ) KMAX = K
 120      CONTINUE
          IF ( KMAX.NE.J ) THEN
            DO 130 I = 1,M
              TEMP = A(I,J)
              A(I,J) = A(I,KMAX)
              A(I,KMAX) = TEMP
 130        CONTINUE
            RDIAG(KMAX) = RDIAG(J)
            WA(KMAX) = WA(J)
            K = IPVT(J)
            IPVT(J) = IPVT(KMAX)
            IPVT(KMAX) = K
          ENDIF
        ENDIF
c
c        compute the householder transformation to reduce the
c        j-th column of a to a multiple of the j-th unit vector.
c
        AJNORM = ENORM(M-J+1,A(J,J))
        IF ( AJNORM.NE.ZERO ) THEN
          IF ( A(J,J).LT.ZERO ) AJNORM = -AJNORM
          DO 140 I = J,M
            A(I,J) = A(I,J)/AJNORM
 140      CONTINUE
          A(J,J) = A(J,J) + ONE
c
c        apply the transformation to the remaining columns
c        and update the norms.
c
          JP1 = J + 1
          IF ( N.GE.JP1 ) THEN
            DO 160 K = JP1,N
              SUM = ZERO
              DO 145 I = J,M
                SUM = SUM + A(I,J)*A(I,K)
 145          CONTINUE
              TEMP = SUM/A(J,J)
              DO 150 I = J,M
                A(I,K) = A(I,K) - TEMP*A(I,J)
 150          CONTINUE
              IF ( .NOT.(.NOT.PIVOT .OR. RDIAG(K).EQ.ZERO) ) THEN
                TEMP = A(J,K)/RDIAG(K)
                RDIAG(K) = RDIAG(K)*SQRT(AMAX1(ZERO,ONE-TEMP**2))
                IF ( P05*(RDIAG(K)/WA(K))**2.LE.EPSMCH ) THEN
                  RDIAG(K) = ENORM(M-J,A(JP1,K))
                  WA(K) = RDIAG(K)
                ENDIF
              ENDIF
 160        CONTINUE
          ENDIF
        ENDIF
        RDIAG(J) = -AJNORM
 200  CONTINUE
      RETURN
c
c     last card of subroutine qrfac.
c
      END
