 
      SUBROUTINE QRSOLV(N,R,LDR,IPVT,DIAG,QTB,X,SDIAG,WA)
      INTEGER N,LDR
      INTEGER IPVT(N)
      REAL R(LDR,N),DIAG(N),QTB(N),X(N),SDIAG(N),WA(N)
c     **********
c
c     subroutine qrsolv
c
c     given an m by n matrix a, an n by n diagonal matrix d,
c     and an m-vector b, the problem is to determine an x which
c     solves the system
c
c           a*x = b ,     d*x = 0 ,
c
c     in the least squares sense.
c
c     this subroutine completes the solution of the problem
c     if it is provided with the necessary information from the
c     qr factorization, with column pivoting, of a. that is, if
c     a*p = q*r, where p is a permutation matrix, q has orthogonal
c     columns, and r is an upper triangular matrix with diagonal
c     elements of nonincreasing magnitude, then qrsolv expects
c     the full upper triangle of r, the permutation matrix p,
c     and the first n components of (q transpose)*b. the system
c     a*x = b, d*x = 0, is then equivalent to
c
c                  t       t
c           r*z = q *b ,  p *d*p*z = 0 ,
c
c     where x = p*z. if this system does not have full rank,
c     then a least squares solution is obtained. on output qrsolv
c     also provides an upper triangular matrix s such that
c
c            t   t               t
c           p *(a *a + d*d)*p = s *s .
c
c     s is computed within qrsolv and may be of separate interest.
c
c     the subroutine statement is
c
c       subroutine qrsolv(n,r,ldr,ipvt,diag,qtb,x,sdiag,wa)
c
c     where
c
c       n is a positive integer input variable set to the order of r.
c
c       r is an n by n array. on input the full upper triangle
c         must contain the full upper triangle of the matrix r.
c         on output the full upper triangle is unaltered, and the
c         strict lower triangle contains the strict upper triangle
c         (transposed) of the upper triangular matrix s.
c
c       ldr is a positive integer input variable not less than n
c         which specifies the leading dimension of the array r.
c
c       ipvt is an integer input array of length n which defines the
c         permutation matrix p such that a*p = q*r. column j of p
c         is column ipvt(j) of the identity matrix.
c
c       diag is an input array of length n which must contain the
c         diagonal elements of the matrix d.
c
c       qtb is an input array of length n which must contain the first
c         n elements of the vector (q transpose)*b.
c
c       x is an output array of length n which contains the least
c         squares solution of the system a*x = b, d*x = 0.
c
c       sdiag is an output array of length n which contains the
c         diagonal elements of the upper triangular matrix s.
c
c       wa is a work array of length n.
c
c     subprograms called
c
c       fortran-supplied ... abs,sqrt
c
c     argonne national laboratory. minpack project. march 1980.
c     burton s. garbow, kenneth e. hillstrom, jorge j. more
c
c     **********
      INTEGER I,J,JP1,K,KP1,L,NSING
      REAL COS,COTAN,P5,P25,QTBPJ,SIN,SUM,TAN,TEMP,ZERO
      SAVE P5,P25,ZERO
      DATA P5,P25,ZERO/5.0E-1,2.5E-1,0.0E0/
c
c     copy r and (q transpose)*b to preserve input and initialize s.
c     in particular, save the diagonal elements of r in x.
c
      DO 100 J = 1,N
        DO 50 I = J,N
          R(I,J) = R(J,I)
 50     CONTINUE
        X(J) = R(J,J)
        WA(J) = QTB(J)
 100  CONTINUE
c
c     eliminate the diagonal matrix d using a givens rotation.
c
      DO 200 J = 1,N
c
c        prepare the row of d to be eliminated, locating the
c        diagonal element using p from the qr factorization.
c
        L = IPVT(J)
        IF ( DIAG(L).NE.ZERO ) THEN
          DO 120 K = J,N
            SDIAG(K) = ZERO
 120      CONTINUE
          SDIAG(J) = DIAG(L)
c
c        the transformations to eliminate the row of d
c        modify only a single element of (q transpose)*b
c        beyond the first n, which is initially zero.
c
          QTBPJ = ZERO
          DO 140 K = J,N
c
c           determine a givens rotation which eliminates the
c           appropriate element in the current row of d.
c
            IF ( SDIAG(K).NE.ZERO ) THEN
              IF ( ABS(R(K,K)).GE.ABS(SDIAG(K)) ) THEN
                TAN = SDIAG(K)/R(K,K)
                COS = P5/SQRT(P25+P25*TAN**2)
                SIN = COS*TAN
              ELSE
                COTAN = R(K,K)/SDIAG(K)
                SIN = P5/SQRT(P25+P25*COTAN**2)
                COS = SIN*COTAN
              ENDIF
c
c           compute the modified diagonal element of r and
c           the modified element of ((q transpose)*b,0).
c
              R(K,K) = COS*R(K,K) + SIN*SDIAG(K)
              TEMP = COS*WA(K) + SIN*QTBPJ
              QTBPJ = -SIN*WA(K) + COS*QTBPJ
              WA(K) = TEMP
c
c           accumulate the tranformation in the row of s.
c
              KP1 = K + 1
              IF ( N.GE.KP1 ) THEN
                DO 122 I = KP1,N
                  TEMP = COS*R(I,K) + SIN*SDIAG(I)
                  SDIAG(I) = -SIN*R(I,K) + COS*SDIAG(I)
                  R(I,K) = TEMP
 122            CONTINUE
              ENDIF
            ENDIF
 140      CONTINUE
        ENDIF
c
c        store the diagonal element of s and restore
c        the corresponding diagonal element of r.
c
        SDIAG(J) = R(J,J)
        R(J,J) = X(J)
 200  CONTINUE
c
c     solve the triangular system for z. if the system is
c     singular, then obtain a least squares solution.
c
      NSING = N
      DO 300 J = 1,N
        IF ( SDIAG(J).EQ.ZERO .AND. NSING.EQ.N ) NSING = J - 1
        IF ( NSING.LT.N ) WA(J) = ZERO
 300  CONTINUE
      IF ( NSING.GE.1 ) THEN
        DO 350 K = 1,NSING
          J = NSING - K + 1
          SUM = ZERO
          JP1 = J + 1
          IF ( NSING.GE.JP1 ) THEN
            DO 310 I = JP1,NSING
              SUM = SUM + R(I,J)*WA(I)
 310        CONTINUE
          ENDIF
          WA(J) = (WA(J)-SUM)/SDIAG(J)
 350    CONTINUE
      ENDIF
c
c     permute the components of z back to components of x.
c
      DO 400 J = 1,N
        L = IPVT(J)
        X(L) = WA(J)
 400  CONTINUE
      RETURN
c
c     last card of subroutine qrsolv.
c
      END
